package com.huasheng.quant.utils;

import org.apache.commons.lang3.StringUtils;

import java.util.regex.Pattern;

public final class MobileValidator {

    public final static String COUNTRY_CODE_CHN = "0086";
    public final static String COUNTRY_CODE_HKG = "0852";
    public final static String COUNTRY_CODE_MAC = "0853";
    public final static String COUNTRY_CODE_TWN = "0886";
    public final static String COUNTRY_CODE_SGP = "0065";

    public final static String COUNTRY_CODE_CHN_STR = "CHN";
    public final static String COUNTRY_CODE_HKG_STR = "HKG";
    public final static String COUNTRY_CODE_MAC_STR = "MAC";
    public final static String COUNTRY_CODE_TWN_STR = "TWN";
    public final static String COUNTRY_CODE_SGP_STR = "SGP";

    private final static Pattern PATTERN_SIMPLE = Pattern.compile("^\\d{5,15}$");

    /**
     * 大陆11位手机号码校验正则<br>
     * 忽略 1349/174 卫星号段, 140/141/144/146/148 13位物联网号段<br>
     * <p>
     * 2020-05-13 把匹配规则改得很 粗暴
     */
    private final static Pattern PATTERN_CHN = Pattern.compile("^1(3\\d|4\\d|5\\d|6[23567]|7\\d|8\\d|9\\d)\\d{8}$");
    private final static Pattern PATTERN_CHN_FULL = Pattern.compile("^(\\+?86)?1(3\\d|4\\d|5\\d|6[23567]|7\\d|8\\d|9\\d)\\d{8}$");

    /**
     * 香港8位手机号码校验正则，区号为 +852<br>
     * 大致有 4/5/6/7/8/9 起头的号段<br>
     * 51-57 && 59<br>
     * 6010-6999<br>
     * 9010-9899<br>
     * <p>
     * 2019-03-07 添加 3/4/7/8，并简化验证
     */
//	private final static Pattern PATTERN_HKG = Pattern.compile("^(5[1-7|9]\\d{6})|([69]0[1-9]\\d{5})|(6[1-9]\\d{6})|(7\\d{7})|(9[1-8]\\d{6})$");
//	private final static Pattern PATTERN_HKG_FULL = Pattern.compile("^(\\+?852)?((5[1-7|9]\\d{6})|([69]0[1-9]\\d{5})|(6[1-9]\\d{6})|(7\\d{7})|(9[1-8]\\d{6}))$");
    private final static Pattern PATTERN_HKG = Pattern.compile("^[3-9]\\d{7}$");
    private final static Pattern PATTERN_HKG_FULL = Pattern.compile("^(\\+?852)?[3-9]\\d{7}$");

    /**
     * 澳門 以 62/63/65/66/68 开头的 8位 手机号码校验正则，区号为 +853<br>
     * {@link https://zh.wikipedia.org/wiki/%E6%BE%B3%E9%96%80%E9%9B%BB%E8%A9%B1%E8%99%9F%E7%A2%BC}<br>
     * {@link https://telecommunications.ctt.gov.mo/web/tc/generalinfo/allocatednum }<br>
     * 660-669 / 688 / 620 623 628 629
     */
//	private final static Pattern PATTERN_MAC = Pattern.compile("^6(6\\d|88|20|23|28|29)\\d{5}$");
//	private final static Pattern PATTERN_MAC_FULL = Pattern.compile("^(\\+?853)?6(6\\d|88|20|23|28|29)\\d{5}$");
    private final static Pattern PATTERN_MAC = Pattern.compile("^6(2|3|5|6|8)\\d{6}$");
    private final static Pattern PATTERN_MAC_FULL = Pattern.compile("^(\\+?853)?6(2|3|5|6|8)\\d{6}$");

    /**
     * 台湾 以 09 开头的 10位 手机号码校验正则，区号为 +886<br>
     * {@link https://www.chahaoba.com/%E5%8F%B0%E6%B9%BE}<br>
     * 拨打时 开头的 0 在某些情况下可以不打
     */
    private final static Pattern PATTERN_TWN = Pattern.compile("^0?9\\d{8}$");
    private final static Pattern PATTERN_TWN_FULL = Pattern.compile("^(\\+?886)?0?9\\d{8}$");

    /**
     * 新加坡8位手机号码校验正则，区号为 +65<br>
     * {@link https://www.chahaoba.com/%E6%96%B0%E5%8A%A0%E5%9D%A1}<br>
     * 大致有 8/9 起头的号段<br>
     * 有 3开头的 Voice Over IP 网络电话<br>
     * 有 6开头的 固定电话
     */
    private final static Pattern PATTERN_SGP = Pattern.compile("^[89]\\d{7}$");
    private final static Pattern PATTERN_SGP_FULL = Pattern.compile("^(\\+?65)?[89]\\d{7}$");

    public static boolean isCNCode(String code) {
        return COUNTRY_CODE_CHN.equals(code) || COUNTRY_CODE_CHN_STR.equals(code);
    }

    public static boolean isHKCode(String code) {
        return COUNTRY_CODE_HKG.equals(code) || COUNTRY_CODE_HKG_STR.equals(code);
    }

    public static boolean isMACCode(String code) {
        return COUNTRY_CODE_MAC.equals(code) || COUNTRY_CODE_MAC_STR.equals(code);
    }

    public static boolean isTWNCode(String code) {
        return COUNTRY_CODE_TWN.equals(code) || COUNTRY_CODE_TWN_STR.equals(code);
    }

    public static boolean isSGPCode(String code) {
        return COUNTRY_CODE_SGP.equals(code) || COUNTRY_CODE_SGP_STR.equals(code);
    }

    /**
     * 判断是否 有限的，可校验的 国籍地区代码
     *
     * @param code
     * @return
     */
    public static boolean isLimitedCode(String code) {
        if (StringUtils.isBlank(code)) {
            return false;
        }
        return isCNCode(code) || isHKCode(code) || isMACCode(code) || isTWNCode(code) || isSGPCode(code);
    }

//	/**
//	 * @deprecated
//	 * @param mobile
//	 * @return
//	 */
//	public static boolean isMobile(String mobile) {
//		return validateFull(mobile) || validateHKFull(mobile);
//	}

    public static boolean isMobile(String mobile, String code) {
        if (StringUtils.isBlank(mobile) || StringUtils.isBlank(code)) {
            return false;
        }
        if (isCNCode(code)) {
            return validate(mobile);
        }
        if (isHKCode(code)) {
            return validateHK(mobile);
        }
        if (isMACCode(code)) {
            return validateMAC(mobile);
        }
        if (isTWNCode(code)) {
            return validateTWN(mobile);
        }
        if (isSGPCode(code)) {
            return validateSGP(mobile);
        }
        return PATTERN_SIMPLE.matcher(mobile).matches();
    }

    public static boolean isCNMobile(String mobile) {
        return validateFull(mobile);
    }

    public static boolean isCNMobile(String mobile, boolean withCode) {
        if (!withCode) {
            return validate(mobile);
        } else {
            return validateFull(mobile);
        }
    }

    public static boolean isHKMobile(String mobile) {
        return validateHKFull(mobile);
    }

    public static boolean isHKMobile(String mobile, boolean withCode) {
        if (!withCode) {
            return validateHK(mobile);
        } else {
            return validateHKFull(mobile);
        }
    }

    public static boolean validateSimple(String mobile, int... sizes) {

        if (!StringUtils.isNumeric(mobile)) {
            return false;
        }

        if (sizes == null || sizes.length == 0) {
            return false;
        }

        boolean test = false;
        for (int s : sizes) {
            if (Integer.compareUnsigned(mobile.length(), s) == 0) {
                test = true;
                break;
            }
        }
        return test;
    }

    /**
     * 验证手机号码 for CHN
     * <p>
     * 手机号码
     *
     * @return
     */
    public static boolean validate(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return false;
        }
        return PATTERN_CHN.matcher(mobile).matches();
    }

    /**
     * 验证手机号码, 包含区号
     *
     * @param str 手机号码
     * @return
     */
    public static boolean validateFull(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return false;
        }
        return PATTERN_CHN_FULL.matcher(mobile).matches();
    }

    /**
     * 校验香港手机号码
     *
     * @param mobile 香港手机号码
     * @return
     */
    public static boolean validateHK(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return false;
        }
        return PATTERN_HKG.matcher(mobile).matches();
    }

    /**
     * 校验香港手机号码, 包含区号
     *
     * @param mobile 香港手机号码
     * @return
     */
    public static boolean validateHKFull(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return false;
        }
        return PATTERN_HKG_FULL.matcher(mobile).matches();
    }

    /**
     * 校验澳门手机号码
     *
     * @param mobile 澳门手机号码
     * @return
     */
    public static boolean validateMAC(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return false;
        }
        return PATTERN_MAC.matcher(mobile).matches();
    }

    /**
     * 校验澳门手机号码, 包含区号
     *
     * @param mobile 澳门手机号码
     * @return
     */
    public static boolean validateMACFull(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return false;
        }
        return PATTERN_MAC_FULL.matcher(mobile).matches();
    }

    /**
     * 校验台湾手机号码
     *
     * @param mobile 台湾手机号码
     * @return
     */
    public static boolean validateTWN(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return false;
        }
        return PATTERN_TWN.matcher(mobile).matches();
    }

    /**
     * 校验台湾手机号码, 包含区号
     *
     * @param mobile 台湾手机号码
     * @return
     */
    public static boolean validateTWNFull(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return false;
        }
        return PATTERN_TWN_FULL.matcher(mobile).matches();
    }

    /**
     * 校验新加坡手机号码
     *
     * @param mobile 新加坡手机号码
     * @return
     */
    public static boolean validateSGP(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return false;
        }
        return PATTERN_SGP.matcher(mobile).matches();
    }

    /**
     * 校验新加坡手机号码, 包含区号
     *
     * @param mobile 新加坡手机号码
     * @return
     */
    public static boolean validateSGPFull(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return false;
        }
        return PATTERN_SGP_FULL.matcher(mobile).matches();
    }

    /**
     * 手机号码信息屏蔽
     *
     * @param mobile
     * @return
     */
    public static String closed(String mobile) {
        if (StringUtils.isBlank(mobile) || mobile.length() <= 4) {
            return mobile;
        }

        int length = mobile.length() - 4;
        int endIndex = length / 2;
        int beginIndex = length / 2 + 4;

        return mobile.substring(0, endIndex) + "****" + mobile.substring(beginIndex, mobile.length());
    }

    /**
     * 手机号码信息屏蔽
     * 大陆电话(11位)：从第4位开始隐藏，隐藏中间4位
     * 香港电话(8位)：隐藏后4位
     *
     * @param mobile
     * @return
     */
    public static String hide(String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return null;
        }

        StringBuilder sb = new StringBuilder();
        if (mobile.length() == 11) {
            sb.append(mobile.substring(0, 3)).append("****").append(mobile.substring(7));
        } else if (mobile.length() > 4) {
            sb.append(mobile.substring(0, mobile.length() - 4)).append("****");
        } else {
            sb.append(mobile);
        }
        return sb.toString();
    }


}
